<?php

class SplendidSpeed {
	/**
	 * Calls register_services once plugins have been loaded.
	 * 
	 * @since 1.0
	 */
	public function init() {
		add_action('plugins_loaded', [$this, 'register_services']);
	}

	/**
	 * Registers all actions and filters needed for the plugin to work.
	 * 
	 * @since 1.0
	 */
	public function register_services() {
		/**
		 * On admin init, check if we have gzip enabled in settings
		 * and if we do then attempt ot add gzip support. If not, then attempt to
		 * remove gzip support.
		 *
		 * @since 1.0
		 */
		add_action('admin_init', function() {
			if($this->post('splendid_form') && $this->get('page') === 'splendid-speed') {
				$this->update_settings();
			}
		});

		/**
		 * Create the admin page.
		 *
		 * @since 1.0
		 */
		add_action('admin_menu', function() {
			add_options_page(
				esc_html__( 'Splendid Speed', 'splendid-speed' ),
				esc_html__( 'Splendid Speed', 'splendid-speed' ),
				'manage_options',
				'splendid-speed',
				[$this, 'admin_page']
			);
		});

		/**
		 * Add an action for deleting all post revisions, clearing up database space.
		 * 
		 * @since 1.0
		 */
		add_action('splendid_speed_weekly_clean_post_revisions', [$this, 'clean_post_revisions']);

		/**
		 * Add an action for deleting all transients, clearing up database  space.
		 * 
		 * @since 1.0
		 */
		add_action('splendid_speed_weekly_clean_transients', [$this, 'clean_transients']);

		/**
		 * Add our admin css and scripts.
		 *
		 * @since 1.0
		 */
		add_action('admin_enqueue_scripts', function($hook) {
		   if($hook === 'settings_page_splendid-speed') {
			   wp_enqueue_style('splendid-speed', SPLENDID_SPEED_DIR_URL . 'assets/css/admin.css');
			   wp_enqueue_script('splendid-speed-admin-page', SPLENDID_SPEED_DIR_URL . 'assets/js/admin.js', [], '1.0', true);
		   }
		});

		/**
		 * Remove the footer text for extra cleanliness.
		 *
		 * @since 1.0
		 */
		add_filter('admin_footer_text', function() {
			$screen = get_current_screen();

			if($screen->id === 'settings_page_splendid-speed') {
				return '<p class="sp-footnote">A very <a class="sp-logo-mark" href="https://splendidpress.com" target="_blank"></a> plugin</p>';
			}
		});

		/**
		 * Register the deactivation hook.
		 * 
		 * @since 1.0
		 */
		register_deactivation_hook(SPLENDID_SPEED_FILE, [$this, 'clean']);

		/**
		 * Add filter for creating a "Settings" link in the plugins page
		 *
		 * @since 1.0
		 */
		add_filter('plugin_action_links_' . SPLENDID_SPEED_BASENAME, [$this, 'plugins_page_link']);

		/**
		 * When instant website load is turned on, put the
		 * instant.page.js in the footer and let it do
		 * its magic.
		 *
		 * @since 1.0
		 */
		if (!is_admin() && $this->setting('instantWebsiteLoad')) {
			add_action('wp_enqueue_scripts', function() {
				wp_enqueue_script('splendid-speed-instant-page', SPLENDID_SPEED_DIR_URL . 'assets/js/instant.page.js', [], '5.1.0', true);
			});
		}

		/**
		 * When instant admin load is turned on, put the
		 * instant.page.js in the footer and let it do
		 * its magic.
		 *
		 * @since 1.0
		 */
		if(is_admin() && $this->setting('instantAdminLoad')) {
			add_action('admin_enqueue_scripts', function() {
				wp_enqueue_script('splendid-speed-instant-page', SPLENDID_SPEED_DIR_URL . 'assets/js/instant.page.js', [], '5.1.0', true);
			});
		}
	}

	/**
	 * Checks for an item in $_POST.
	 * 
	 * @return bool
	 * 
	 * @since 1.0
	 */
	private function post($key) {
		if(!empty($_POST[$key])) {
			return true;
		}

		return false;
	}

	/**
	 * checks for an item in $_GET.
	 * 
	 * @return string|bool 
	 *
	 * @since 1.0
	 */
	private function get($key) {
		if(!empty($_GET[$key])) {
			return sanitize_key($_GET[$key]);
		}

		return false;
	}

	/**
	 * Returns all of the settings.
	 *
	 * @return array
	 *
	 * @since 1.0
	 */
	public function settings() {
		return get_option('splendid_speed_settings', []);
	}

	/**
	 * Returns a specific item from `sp_settings()`.
	 *
	 * @param $key
	 * @return false|mixed
	 *
	 * @since 1.0
	 */
	public function setting($key) {
		$settings = $this->settings();

		if(!empty($settings[$key])) {
			return $settings[$key];
		}

		return false;
	}

	/**
	 * Updates the settings by given key => value collection.
	 *
	 * @param $arr
	 *
	 * @since 1.0
	 */
	public function update_settings() {
		$settings = [];

		// Preload website pages
		if($this->post('instantWebsiteLoad')) {
			$settings['instantWebsiteLoad'] = true;
		}

		// Preload admin pages
		if($this->post('instantAdminLoad')) {
			$settings['instantAdminLoad'] = true;
		}

		// Gzip static assets
		if($this->post('gzip')) {
			$settings['gzip'] = true;
			$this->gzip_add();
		} else {
			$this->gzip_remove();
		}

		// Leverage browser caching
		if($this->post('browser_caching')) {
			$settings['browser_caching'] = true;
			$this->browser_caching_add();
		} else {
			$this->browser_caching_remove();
		}

		// Clean post revisions
		if($this->post('clean_post_revisions')) {
			$settings['clean_post_revisions'] = true;
			$this->clean_post_revisions();
			wp_schedule_event(time(), 'weekly', 'sp_daily_clean_post_revisions');
		} else {
			wp_clear_scheduled_hook('sp_daily_clean_post_revisions');
		}

		// Clean transients
		if($this->post('clean_transients')) {
			$settings['clean_transients'] = true;
			$this->clean_transients();
			wp_schedule_event(time(), 'weekly', 'sp_daily_clean_transients');
		} else {
			wp_clear_scheduled_hook('sp_daily_clean_transients');
		}

		update_option('splendid_speed_settings', $settings);
	}

	/**
	 * Adds the needed rules to .htaccess to enable
	 * gzipping of static assets.
	 *
	 * @return bool
	 *
	 * @since 1.0
	 */
	private function gzip_add() {
		$lines = [
			'<IfModule mod_deflate.c>',
			'AddOutputFilterByType DEFLATE image/svg+xml',
			'AddOutputFilterByType DEFLATE application/javascript',
			'AddOutputFilterByType DEFLATE application/rss+xml',
			'AddOutputFilterByType DEFLATE application/vnd.ms-fontobject',
			'AddOutputFilterByType DEFLATE application/x-font',
			'AddOutputFilterByType DEFLATE application/x-font-opentype',
			'AddOutputFilterByType DEFLATE application/x-font-otf',
			'AddOutputFilterByType DEFLATE application/x-font-truetype',
			'AddOutputFilterByType DEFLATE application/x-font-ttf',
			'AddOutputFilterByType DEFLATE application/x-javascript',
			'AddOutputFilterByType DEFLATE application/xhtml+xml',
			'AddOutputFilterByType DEFLATE application/xml',
			'AddOutputFilterByType DEFLATE font/opentype',
			'AddOutputFilterByType DEFLATE font/otf',
			'AddOutputFilterByType DEFLATE font/ttf',
			'AddOutputFilterByType DEFLATE image/x-icon',
			'AddOutputFilterByType DEFLATE text/css',
			'AddOutputFilterByType DEFLATE text/html',
			'AddOutputFilterByType DEFLATE text/javascript',
			'AddOutputFilterByType DEFLATE text/plain',
			'AddOutputFilterByType DEFLATE text/xml',
			'</IfModule>'
		];

		return insert_with_markers(get_home_path() . '.htaccess', 'Splendid Speed GZIP', $lines);
	}

	/**
 	* Removes the gzipping rules from .htaccess.
	 *
	 * @return bool
	 *
	 * @since 1.0
	 */
	private function gzip_remove() {
		return insert_with_markers(get_home_path() . '.htaccess', 'Splendid Speed GZIP', []);
	}

	/**
	 * Tests whether or not the gzipping works.
	 * The usual reasons it won't work is if .htaccess is not writable, or
	 * we're dealing with a apache setting that doesn't have mod_deflate.
	 *
	 * It can also happen that the server is actually nginx, in which case enabling
	 * gzipping can not be done.
	 *
	 * @return bool
	 *
	 * @since 1.0
	 */
	public function gzip_test_passes() {
		$res = wp_remote_get(get_site_url() . '?time=' . time(), [
			'headers' => ['Content-Encoding' => 'gzip'],
			'sslverify' => false
		]);

		if(!is_array($res)) {
			return false;
		}

		return strpos($res['headers']['content-encoding'], 'gzip') !== false;
	}

	/**
	 * Adds the needed rules to .htaccess to allow
	 * caching of static assets.
	 *
	 * @return bool
	 *
	 * @since 1.0
	 */
	private function browser_caching_add() {
		$lines = [
			'<IfModule mod_expires.c>',
			'ExpiresActive on',
			'ExpiresDefault "access plus 2 days"',
			'ExpiresByType image/jpg "access plus 1 month"',
			'ExpiresByType image/svg+xml "access 1 month"',
			'ExpiresByType image/gif "access plus 1 month"',
			'ExpiresByType image/jpeg "access plus 1 month"',
			'ExpiresByType image/png "access plus 1 month"',
			'ExpiresByType text/css "access plus 1 month"',
			'ExpiresByType text/javascript "access plus 1 month"',
			'ExpiresByType application/javascript "access plus 1 month"',
			'ExpiresByType application/x-shockwave-flash "access plus 1 month"',
			'ExpiresByType image/ico "access plus 1 month"',
			'ExpiresByType image/x-icon "access plus 1 month"',
			'ExpiresByType text/html "access plus 600 seconds"',
			'</IfModule>'
		];

		return insert_with_markers(get_home_path() . '.htaccess', 'Splendid Speed Cache', $lines);
	}

	/**
	 * Removes the caching rules from .htaccess.
	 *
	 * @return bool
	 *
	 * @since 1.0
	 */
	private function browser_caching_remove() {
		return insert_with_markers(get_home_path() . '.htaccess', 'Splendid Speed Cache', []);
	}

	/**
	 * Cleans all post revisions, clearing up database space.
	 *
	 * @since 1.0
	 */
	public function clean_post_revisions() {
		global $wpdb;
		$sql = "DELETE FROM " . $wpdb->prefix . "posts WHERE post_type = %s;";
		$wpdb->query($wpdb->prepare($sql, 'revision'));
	}

	/**
	 * Cleans all transients, clearing up database  space.
	 *
	 * @since  1.0
	 */
	public function clean_transients() {
		global $wpdb;

		$sql = "DELETE FROM " .  $wpdb->prefix . "options WHERE option_name LIKE ('_transient_%')";
		$wpdb->query($sql);

		$sql = "DELETE FROM " .  $wpdb->prefix . "options WHERE option_name LIKE ('_site_transient_%')";
		$wpdb->query($sql);
	}

	/**
	 * Deactivation cleaning. Because causing more
	 * bloat to your website is the last thing I want.
	 *
	 * @since 1.0
	 */
	public function clean() {
		// Remove gzipping of static assets, if used.
		if($this->setting('gzip')) {
			$this->gzip_remove();
		}

		// Remove caching of static assets, if used.
		if($this->setting('browser_caching')) {
			$this->browser_caching_remove();
		}

		// Remove scheduled event for post revisions deletion, if used.
		if($this->setting('clean_post_revisions')) {
			wp_clear_scheduled_hook('splendid_speed_weekly_clean_post_revisions');
		}

		// Remove scheduled event for transients deletion, if used.
		if($this->setting('clean_transients')) {
			wp_clear_scheduled_hook('splendid_speed_weekly_clean_transients');
		}

		// And finally, delete all options.
		delete_option('splendid_speed_settings');
	}

	/**
	 * Adds a "Settings" link to the plugin screen for Splendid's
	 * plugin, that way the user can more easily find the settings
	 * once the plugin is activated.
	 * 
	 * @since 1.0
	 */
	public function plugins_page_link($links) {
		$url = get_admin_url() . "options-general.php?page=splendid-speed";
		$settings_link = '<a href="' . $url . '">' . __('Settings', 'splendid-speed') . '</a>';
		array_unshift($links, $settings_link);

		return $links;
	}

	/**
	 * Renders the admin page.
	 * 
	 * @since 1.0
	 */
	public function admin_page() {
		$instantWebsiteLoad = $this->setting('instantWebsiteLoad');
		$instantAdminLoad = $this->setting('instantAdminLoad');
		$gzip = $this->setting('gzip');
		$gzip_works = $this->gzip_test_passes();
		$browser_caching = $this->setting('browser_caching');
		$clean_post_revisions = $this->setting('clean_post_revisions');
		$clean_transients = $this->setting('clean_transients');
		?>
		<div class="sp-header">
			<div class="sp-logo"></div>
		</div>
		<div class="sp-body">	
			<form method="post" id="splendid-speed">
				<input type="hidden" name="splendid_form" value="1">

				<!-- Preload website pages -->
				<div class="sp-option <?php if($instantWebsiteLoad): ?>sp-option-active<?php endif; ?>">
					<div class="sp-option-heading">
						<h3><?php _e('Preload website pages', 'splendid-speed'); ?>
							<?php if($instantWebsiteLoad): ?>
								<span><?php _e('Active', 'splendid-speed'); ?></span>
							<?php else: ?>
								<span><?php _e('Inactive', 'splendid-speed'); ?></span>
							<?php endif; ?>
						</h3>
					</div>
					<div class="sp-option-description">
						<label class="switch">
							<input type="checkbox" name="instantWebsiteLoad" <?php if($instantWebsiteLoad): ?>checked<?php endif; ?>>
							<span class="slider"></span>
							<p>
						<?php _e('Preload the pages of your WordPress website right before you click on a link. Gives you
						much faster website loading which enhances user experience.', 'splendid-speed'); ?></p></label>
					</div>
				</div>

				<!-- Preload admin pages -->
				<div class="sp-option <?php if($instantAdminLoad): ?>sp-option-active<?php endif; ?>">
					<div class="sp-option-heading">
						<h3><?php _e('Preload admin pages', 'splendid-speed'); ?>
							<?php if($instantAdminLoad): ?>
								<span><?php _e('Active', 'splendid-speed'); ?></span>
							<?php else: ?>
								<span><?php _e('Inactive', 'splendid-speed'); ?></span>
							<?php endif; ?>
						</h3>
					</div>
					<div class="sp-option-description">
						<label class="switch">
							<input type="checkbox" name="instantAdminLoad" <?php if($instantAdminLoad): ?>checked<?php endif; ?>>
							<span class="slider"></span>
							<p>
						<?php _e('Preload the pages of your WordPress admin panel right before you click on a link. Gives you much faster admin panel loading which enhances user experience.', 'splendid-speed'); ?></p></label>
					</div>
				</div>

				<!--  ZIP static assets -->
				<div class="sp-option <?php if($gzip || $gzip_works): ?>sp-option-active<?php endif; ?>">
					<div class="sp-option-heading">
						<h3><?php _e('ZIP static assets', 'splendid-speed'); ?>
							<?php if($gzip || $gzip_works): ?>
								<span><?php _e('Active', 'splendid-speed'); ?></span>
							<?php else: ?>
								<span><?php _e('Inactive', 'splendid-speed'); ?></span>
							<?php endif; ?>
						</h3>
					</div>
					<div class="sp-option-description">
						<label class="switch">
							<input type="checkbox" name="gzip" <?php if($gzip || $gzip_works): ?>checked<?php endif; ?>>
							<span class="slider"></span>
							<p>
						<?php _e('Zip static assets with Gzip technology to reduce the size of your static assets, such as images, stylesheets and scripts. Smaller sizes of these assets allow the browser to load your website faster.', 'splendid-speed'); ?></p></label>
					</div>
					<?php if($gzip && !$gzip_works): ?>
						<div class="sp-option-info-description">
							<span class="dashicons dashicons-warning"></span>
							<?php _e('Zipping does not work. This is most likely either because .htaccess file is not writable, your web server does not have mod_deflate enabled or you are not running Apache web server at all.', 'splendid-speed'); ?>
						</div>
					<?php endif; ?>
				</div>

				<!-- Cache static assets -->
				<div class="sp-option <?php if($browser_caching): ?>sp-option-active<?php endif; ?>">
					<div class="sp-option-heading">
						<h3><?php _e('Cache static assets', 'splendid-speed'); ?>
							<?php if($browser_caching): ?>
							  	<span><?php _e('Active', 'splendid-speed'); ?></span>
							<?php else: ?>
								<span><?php _e('Inactive', 'splendid-speed'); ?></span>
							<?php endif; ?>
						</h3>
					</div>
					<div class="sp-option-description">
						<label class="switch">
							<input type="checkbox" name="browser_caching" <?php if($browser_caching): ?>checked<?php endif; ?>>
							<span class="slider"></span>
							<p>
						<?php _e('Tell browsers to hold onto static assets such as images, stylesheets and scripts for longer. This will make your pages load faster for people who have already visited your website, because all the static assets have already been downloaded.', 'splendid-speed'); ?></p></label>
					</div>
				</div>

				<!-- Clean post revisions -->
				<div class="sp-option <?php if($clean_post_revisions): ?>sp-option-active<?php endif; ?>">
				  <div class="sp-option-heading">
					<h3><?php _e('Clean post revisions', 'splendid-speed'); ?>
						<?php if($clean_post_revisions): ?>
							<span><?php _e('Active', 'splendid-speed'); ?></span>
						<?php else: ?>
							<span><?php _e('Inactive', 'splendid-speed'); ?></span>
						<?php endif; ?>
					</h3>
				  </div>
				  <div class="sp-option-description">
				  	<label class="switch">
						  <input type="checkbox" name="clean_post_revisions" <?php if($clean_post_revisions): ?>checked<?php endif; ?>>
						  <span class="slider"></span>
					  <p>
					  	<?php _e('Delete all post revisions periodically every week. This helps keep your database
					  size small. Note however that by doing so all old versions of content will disappear for good.', 'splendid-speed'); ?></p>
					  </label>
				  </div>
				</div>

				<!-- Clean transients -->
				<div class="sp-option <?php if($clean_transients): ?>sp-option-active<?php endif; ?>">
				  	<div class="sp-option-heading">
						<h3><?php _e('Clean transients', 'splendid-speed'); ?>
							<?php if($clean_transients): ?>
								<span><?php _e('Active', 'splendid-speed'); ?></span>
							<?php else: ?>
								<span><?php _e('Inactive', 'splendid-speed'); ?></span>
							<?php endif; ?>
						</h3>
				  	</div>
				  <div class="sp-option-description">
					<label class="switch">
						<input type="checkbox" name="clean_transients" <?php if($clean_transients): ?>checked<?php endif; ?>>
						<span class="slider"></span>
						<p>
							<?php _e('Delete all transient information periodically every week. Transients are used to temporarily store cached information by plugin developers and cleaning them helps keep your database small.', 'splendid-speed');  ?>
						</p>
				  </div>
				</div>
			</form>
		</div>
	<?php
	}
}